"use strict";

(function() {

    window.insidePoints = function(x, y, points) {
        var odd = false;
        
        for(var i = 0; i < points.length; i++) {
            var one = points[i];
            var two = points[(i+1)%points.length];
            
            //Confirms that y is between y1 and y2
            if((one[1] > y) !== (two[1] > y)) {
                //Since we know y is between y1 and y2, we know the two lines intersect
                //But we are only interested in the ray pointing to the positive x
                //direction. So we check if the intersection is to towards positive x.
                var t1= one[0] - x + (( (one[1]-y) / (one[1] - two[1]) )*(two[0]-one[0]));
                
                if(t1 > 0) {
                    odd = !odd;
                }
            }
        }
        
        return odd;
    };

    window.convexHull = function(points) {
        var prev, current, lowest;
        var hull = [];
        
        //Find the lowest point and start with that (actually highest point)
        var index = 0;
        current = points[0];
        
        for(var i = 1; i < points.length; i++) {
            if(points[i][1] < current[1]) {
                index = i;
                current = points[i];
            }
        }
        points.splice(index,1);
        hull.push(current);
        lowest = current;
        
        prev = [current[0]-1, current[1]];
        
        while(points.length > 0) {
            var A = subAndNormalize(current, prev);
            
            var endCosine = dot(A, subAndNormalize(lowest, current));        
            
            index = 0;
            var B = subAndNormalize(points[0], current);
            var biggest = dot(A,B);
            
            for(var i = 1; i < points.length; i++) {
                
                var B = subAndNormalize(points[i], current);
                
                if(dot(A,B) > biggest) {
                    index = i;
                    biggest = dot(A,B);
                }
            }
            
            if(hull.length > 1 && endCosine >= biggest) {
                break;
            }
            prev = current;
            current = points[index];
            points.splice(index, 1);
            
            hull.push(current);
        }
        
        return hull;
    };

    function subAndNormalize(first, second) {
        var sub = [first[0]-second[0], first[1]-second[1]];
        var len = Math.sqrt(sub[0]*sub[0] + sub[1]*sub[1]);
        return [sub[0]/len, sub[1]/len];
    }

    function dot(first, second) {
        return first[0]*second[0] + first[1]*second[1];
    }

    window.intersect = function(pointsA, pointsB) {    
        var MTVs = [];
        var index = 0;
        
        var i = pointsA.length - 1;
        var j = 0;
        for(; j < pointsA.length; i = j, j++) {
            //Axis is equal to the perpendicular of an edge
            var axis = [pointsA[j][1] - pointsA[i][1], pointsA[i][0] - pointsA[j][0]];
            
            MTVs[index] = getMTV(axis, pointsA, pointsB);
            
            if(MTVs[index] === null) {
                return null;
            }
            
            index++;
        }
        
        var i = pointsB.length - 1;
        var j = 0;
        for(; j < pointsB.length; i = j, j++) {
            //Axis is equal to the perpendicular of an edge
            var axis = [pointsB[j][1] - pointsB[i][1], pointsB[i][0] - pointsB[j][0]];
            
            MTVs[index] = getMTV(axis, pointsA, pointsB);
            
            if(MTVs[index] === null) {
                return null;
            }
            
            index++;
        }
        
        var minMTV = dot(MTVs[0], MTVs[0]);
        var minIndex = 0;
        for(var i = 1; i < index; i++) {
            var temp = dot(MTVs[i], MTVs[i]);
            
            if(temp < minMTV) {
                minMTV = temp;
                minIndex = i;
            }
        }
        
        return MTVs[minIndex];
    };
    
    //Projects the polygon points onto the axis and returns the MTV.
    //MTV assumes you are pushing pointsA away form pointsB.
    //If the polygons do not overlap then return null
    function getMTV(axis, pointsA, pointsB) {
        var boundsA = calcInterval(axis, pointsA);
        var boundsB = calcInterval(axis, pointsB);
        
        if(boundsA[1] < boundsB[0] || boundsB[1] < boundsA[0])  {
            return null;
        }
        
        var d1 = boundsA[1] - boundsB[0];
        var d2 = boundsB[1] - boundsA[0];
        var d = (d1 < d2) ? -1*d1 : d2;
        
        d = d / dot(axis, axis);
        
        return [axis[0] * d , axis[1] * d];
    }

    function calcInterval(axis, points) {
        var ret = [ dot(axis, points[0]), dot(axis, points[0]) ];

        for(var i = 1; i < points.length; i++) {
            var d = dot(axis, points[i]);
            if(d < ret[0]) {
                ret[0] = d;
            } else if(d > ret[1]) {
                ret[1] = d;
            }
        }
        
        return ret;
    }
    
})();